"""Deduplicate learning rules in the database."""
import sys

from testbed.mldb import get_default_mldb

def main(dry_run=True):
    mldb = get_default_mldb()
    rules_found = dict()
    duplicates = 0
    for curr_rule in mldb.get_learning_rules():
        family = curr_rule.db_obj.subtype
        params = curr_rule.params
        if family not in rules_found:
            rules_found[family] = [curr_rule]
            continue

        found_duplicate = False
        for existing_rule in rules_found[family]:
            if existing_rule.params == params:
                print("=" * 80)
                print('Found duplicate!')
                print("Family: ", family)
                print(existing_rule.params)
                print(params)
                found_duplicate = True
                duplicates += 1

                # Find all of the relationships that this object is a part of
                # and rewrite them so that the existing rule participates
                # in the relationship
                first_rels = mldb.db.get_relationship(first=curr_rule.db_obj)
                for first_rel in first_rels:
                    if dry_run:
                        print("Found relationsip\n", first_rel)
                        print("Replace ", first_rel.first, "->", existing_rule.db_obj.uuid)
                        print("Current uuid", curr_rule.db_obj.uuid)
                    else:
                        mldb.db.insert_relationship(first=existing_rule.db_obj,
                            second=first_rel.second, type_=first_rel.type_)

                second_rels = mldb.db.get_relationship(second=curr_rule.db_obj)
                for second_rel in second_rels:
                    if dry_run:
                        print("Found relationsip\n", second_rel)
                        print("Replace ", second_rel.second, "->", existing_rule.db_obj.uuid)
                        print("Current uuid", curr_rule.db_obj.uuid)
                    else:
                        mldb.db.insert_relationship(first=second_rel.first,
                            second=existing_rule.db_obj, type_=second_rel.type_)

                if dry_run:
                    print("Would delete rule: ", curr_rule.db_obj)
                else:
                    blob_uuids = [blob.uuid for blob in mldb.db.get_blobs(curr_rule.uuid).values()]
                    mldb.db.delete_blob(blob_uuids)
                    mldb.db.delete(curr_rule.db_obj)
                break

        if not found_duplicate:
            rules_found[family].append(curr_rule)

    print(f"Found {duplicates} duplicates!")


if __name__ == '__main__':
    if len(sys.argv) == 2 and sys.argv[1] == "--delete":
        main(dry_run=False)
    elif len(sys.argv) == 1 or (len(sys.argv) == 2 and sys.argv[1] == "--dry-run"):
        main(dry_run=True)
    else:
        raise NotImplementedError
