"""Script to interact with (train/eval) neural models in parallel on a cluster."""
import concurrent.futures
import os
import subprocess
import sys


sys.path.append('../tools')
from cifar_model_perf import chosen_models

MODELS = chosen_models
# MODELS = [
#     # 'LeNet',
#     # 'PNASNetA',
#     # 'ShuffleNetV2_0_5',
#     # 'MobileNetV2',
#     # 'VGG11',
#     # 'GoogLeNet',
#     'densenet_cifar',
#     'PreActResNet18',
#     'ResNet50',
#     'ResNeXt29_4x64d',
# ]
TRAIN_DATASETS = ["cinic10-imgnt-only-all"]
TEST_DATASETS = ["cifar10-test", "cifar10.1-v6", "cifar10.2-test", "cifar10.2-all"]


def train_model(model, dataset):
    command = [
        "sbatch",
        "../run.sh",
        "python",
        "experiment.py",
        "train",
        "--model_family",
        f"{model}",
        "--train_set",
        dataset,
        "--store_early",
        "--num_checkpoints",
        "10",
        "--device",
        "cuda:0",
    ]
    print(' '.join(command))
    subprocess.call(command)


def evaluate_model(model, train_dataset, eval_dataset):
    command = [
        # "sbatch",
        # "../run.sh",
        "python",
        "experiment.py",
        "evaluate",
        "--model_family",
        model,
        "--train_set",
        train_dataset,
        "--test_set_pattern",
        eval_dataset + "$",
        "--eval_all_checkpoints",
    ]
    print(' '.join(command))
    subprocess.call(command)


def run_training():
    for model in MODELS:
        for dataset in TRAIN_DATASETS:
            train_model(model, dataset)


def run_evaluations():
    for model in MODELS:
        for train_dataset in TRAIN_DATASETS:
            for test_dataset in TEST_DATASETS:
                evaluate_model(model, train_dataset, test_dataset)


if __name__ == "__main__":
    assert len(sys.argv) == 2
    if sys.argv[1] == "train":
        run_training()
    elif sys.argv[1] == "eval":
        run_evaluations()
    else:
        print("Must be one of train/eval")
        raise NotImplementedError
