"""Set up learning rules that finetune pretrained ImageNet models on Cifar10"""
import tqdm
import sys
sys.path.append("..")

import testbed
from testbed.mldb import get_default_mldb

print("Loading MLDB")
mldb = get_default_mldb()

neural_model_names = [k for k, v in mldb.model_registry.families.items() if isinstance(v, testbed.model_families.neural_models.NeuralModelFamily)]


print("Getting all Neural model family names")
fam_names = [
    f for f in testbed.model_families.default_registry().families
    if f in neural_model_names]
print(f"Found {len(fam_names)} models")

print("Creating and storing learning rules")
progress_bar = tqdm.tqdm(fam_names)
for fam in progress_bar:
    progress_bar.set_postfix({'network': fam})
    rule = mldb.learning_rule(
        family=fam,
        name='cinic10_train_' + fam,
        params=dict(
            batch_size=128,
            total_examples=210000*300,
            optimizer='SGD',
            lr=0.01,
            momentum=0.9,
            weight_decay=1e-4,
            lr_scheduler='MultiStepLR',
            milestones=(210000*90, 210000*180, 210000*270),
            gamma=0.1,
        )
    )
    rule.store()
    rule = mldb.learning_rule(
        family=fam,
        name='cinic10_train_4gpu_' + fam,
        params=dict(
            batch_size=128*4,
            total_examples=210000*300,
            optimizer='SGD',
            lr=0.01,
            momentum=0.9,
            weight_decay=1e-4,
            lr_scheduler='MultiStepLR',
            milestones=(210000*90, 210000*180, 210000*270),
            gamma=0.1,
        )
    )
    rule.store()
    rule = mldb.learning_rule(
        family=fam,
        name='cinic10_train_short_' + fam,
        params=dict(
            batch_size=128,
            total_examples=210000*40,
            optimizer='SGD',
            lr=0.01,
            momentum=0.9,
            weight_decay=1e-4,
            lr_scheduler='MultiStepLR',
            milestones=(210000*20, 210000*30),
            gamma=0.1,
        )
    )
    rule.store()
