"""Set up learning rules that finetune pretrained ImageNet models on Cifar10"""
import tqdm
import sys
sys.path.append("..")

import testbed
from testbed.mldb import get_default_mldb

print("Loading MLDB")
mldb = get_default_mldb()

print("Getting all ImageNet model family names")
fam_names = [
    f for f in testbed.model_families.default_registry().families
    if f.startswith('imgnt_')]
print(f"Found {len(fam_names)} models")

print("Creating and storing learning rules")
progress_bar = tqdm.tqdm(fam_names)
for fam in progress_bar:
    net_name = fam.replace("imgnt_", "")
    progress_bar.set_postfix({'network': net_name})
    rule = mldb.learning_rule(
        family='imgnt_' + net_name,
        name='imgnt_pretrained_224_' + net_name,
        params=dict(
            num_classes=10,
            input_size=(224, 224),
            pretrained=True,
            batch_size=32,
            epochs=100,
            optimizer='SGD',
            lr=0.01,
            momentum=0.9,
            weight_decay=0.0,
            lr_scheduler='StepLR',
            gamma=0.975,
            dropout_p=0.2,
        )
    )
    rule.store()
    rule = mldb.learning_rule(
        family='imgnt_' + net_name,
        name='imgnt_linear_probe_' + net_name,
        params=dict(
            num_classes=10,
            input_size=(224, 224),
            pretrained=True,
            finetune_only_last_layer=True,
            batch_size=32,
            epochs=100,
            optimizer='SGD',
            lr=0.01,
            momentum=0.9,
            weight_decay=0.0,
            lr_scheduler='StepLR',
            gamma=0.975,
            dropout_p=None,
        )
    )
    rule.store()
