'Base classes for datasets.'
from abc import ABC, abstractmethod
import dataclasses

class DataFamily(ABC):
    @abstractmethod
    def provider(self, data_params):
        pass
    
    @property
    @abstractmethod
    def name(self):
        pass
    
    @property
    @abstractmethod
    def params(self):
        pass


@dataclasses.dataclass
class NoParameters:
    pass


class DataProviderImplementation(ABC):
    @abstractmethod
    def get_dataset(self, mldb, db_obj, show_progress):
        pass
    
    @abstractmethod
    def store_dataset(self, mldb, db_obj, show_progress):
        pass

    @property
    @abstractmethod
    def family(self):
        pass
    
    @property
    @abstractmethod
    def params(self):
        pass

    # Separate method so you can get the size of a dataset
    # without downloading it.
    @abstractmethod
    def __len__(self):
        pass

    def subsample(self, size, seed, mldb, db_obj):
        raise NotImplementedError
    

class DatasetImplementation(ABC):
    @abstractmethod
    def __len__(self):
        pass

    @abstractmethod
    def __getitem__(self, index):
        pass
    
    @property
    def examples(self):
        raise NotImplementedError

    @property
    def labels(self):
        raise NotImplementedError
