import dataclasses
import pickle

import numpy as np

from .. import learning_rule
from .. import model
from .. import model_family


class LinearModel(model.ModelImplementation):
    def __init__(self, weights):
        self.weights = weights
    
    def predict(self, examples):
        preds = examples @ self.weights
        return np.argmax(preds, axis=1), preds
    
    def serialize(self):
        return pickle.dumps(self.weights)


def deserializer(model_bytes):
    weights = pickle.loads(model_bytes)
    return LinearModel(weights)


@dataclasses.dataclass
class Hyperparameters:
    regularizer: float
    num_classes: int = 10


def rule_generator(hparams):
    def cur_learning_rule(dataset):
        return LinearModel(np.random.randn(dataset.examples.shape[1]))
    return learning_rule.SimpleFunctionLearningRule(cur_learning_rule)


# This is going to get called automatically and added to the model family registry
def get_model_families():
    return {
        'lstsq_ova': model_family.SimpleModelFamily(rule_generator, deserializer, Hyperparameters)
    }
