from abc import ABC, abstractmethod
import dataclasses


class ModelFamily(ABC):
    @abstractmethod
    def learning_rule(self, rule_params):
        pass

    @abstractmethod
    def deserialize_model(self, model_bytes):
        pass

    @property
    @abstractmethod
    def rule_params(self):
        pass

    @property
    @abstractmethod
    def train_params(self):
        pass
    
    @property
    @abstractmethod
    def eval_params(self):
        pass


@dataclasses.dataclass
class NoParameters:
    pass


class SimpleModelFamily(ModelFamily):
    def __init__(self,
                 learning_rule_generator,
                 deserializer,
                 rule_parameter_class=NoParameters,
                 train_parameter_class=NoParameters,
                 eval_parameter_class=NoParameters):
        self.learning_rule_generator = learning_rule_generator
        self.deserializer = deserializer
        self.rule_parameters = rule_parameter_class
        self.train_parameters = train_parameter_class
        self.eval_parameters = eval_parameter_class
    
    def learning_rule(self, hparams):
        return self.learning_rule_generator(hparams)
    
    def deserialize_model(self, model_bytes):
        return self.deserializer(model_bytes)

    @property
    def rule_params(self):
        return self.rule_parameters

    @property
    def train_params(self):
        return self.train_parameters
    
    @property
    def eval_params(self):
        return self.eval_parameters
