"""Testing  fast neural learning rule functionality."""
import pytest
import sys

import numpy as np
import torch

from testbed import init_db
from testbed.learning_rule import LearningRule


def test_cpu():
    # silly synthetic data
    n = 100
    y = np.random.choice([0, 1], size=n)
    mu = np.random.randn(1, 32, 32, 3)
    mu = mu / np.linalg.norm(mu, axis=-1, keepdims=True)
    x = np.random.randn(n, 32, 32, 3) / np.sqrt(32 * 32 * 3)
    x += mu * (2 * y.reshape(-1, 1, 1, 1) - 1)
    
    rule = LearningRule(family='myrtlenet',
                        params=dict(
                            batch_size=25,
                            gbn_splits=1,
                            ema_freq=1,
                            lr_max=0.1,
                            ema_momentum=0,
                            total_examples=500))

    # test that network fits data
    model = rule.learn(examples=x, labels=y,
                       params=dict(
                           device='cpu',
                           half_precision=False))
    eval = model.predict(examples=x, labels=y,
                         params=dict(device='cpu'))
    assert eval.accuracy >= 0.9


@pytest.mark.gpu
def test_myrtlenet(mldb):
    init_db.add_cifar10(mldb)

    rule = mldb.learning_rule(family='myrtlenet')

    # test that we get the promise ~94% accuracy
    model = rule.learn(dataset='cifar10-train',
                       half_precision=torch.cuda.is_available(),
                       verbose=True)
    eval = model.predict(dataset='cifar10-test')
    assert eval.accuracy >= 0.93

    # test that tta is hleful
    eval_no_tta = model.predict(dataset='cifar10-test', tta=False)
    assert eval.accuracy > eval_no_tta.accuracy

