"""Tests of featurization functionality."""
import pytest
import sys

import numpy as np

from testbed import dataset
from testbed import model
from testbed.featurization import Featurization


def test_featurization(mldb, rng):
    """Simple tests of evaluation functionality."""
    examples = rng.randn(20, 3)
    labels = np.array([0] * 10 + [1] * 10)

    dataset = mldb.dataset(examples=examples, labels=labels)
    
    rule = mldb.learning_rule(family='debug_1', name='lr1', params={"output_value": 1})
    rule.store()
    model = rule.learn(dataset=dataset)
    featurization = model.featurize(dataset=dataset)

    assert np.allclose(featurization.features, np.full(len(labels), 2))
    
    featurization.store()

    from_cache = Featurization(mldb=mldb, db_obj=featurization.db_obj.uuid)
    assert np.allclose(from_cache.features, np.full(len(labels), 2))

    no_labels_dataset = mldb.dataset(examples=examples)
    featurization2 = model.featurize(dataset=no_labels_dataset)
    assert np.allclose(featurization2.features, np.full(len(labels), 2))


def test_params_stored(mldb, rng):
    rule1 = mldb.learning_rule(family='debug_1')
    dset1 = mldb.dataset(name='dset1', examples=np.array([1, 1]), labels=np.array([2, 2]))
    model1 = rule1.learn(dataset=dset1, name='model1')
    mldb.dataset(name='dset2', examples=np.array([1, 1, 1]), labels=np.array([2, 2, 2])).store()
    featurization1 = model1.featurize(dataset='dset2', output_value=5)
    assert np.array_equal(featurization1.features, np.full(3, 6))
    assert featurization1.params.output_value == 5
    assert len(mldb.get_featurizations()) == 0
    featurization1.store()
    assert len(mldb.get_featurizations()) == 1
    featurization1_again = mldb.get_featurizations()[0]
    assert featurization1_again.params.output_value == 5