"""Testing  neural learning rule functionality."""
import pytest

from testbed import init_db

@pytest.mark.gpu
def test_full_cifar10(mldb, rng):
    init_db.add_odst_models(mldb)
    init_db.add_cifar10(mldb)
    model = mldb.model(name='odst_resnet50_model')
    eval = model.predict(dataset='cifar10-test')
    acc = eval.accuracy
    assert acc >= 0.9811
    assert acc <= 0.9813
    
    model = mldb.model(name='odst_shakedroppyramid272_model')
    eval = model.predict(dataset='cifar10-test')
    acc = eval.accuracy
    assert acc >= 0.9856
    assert acc <= 0.9858


def test_cifar10_subset(mldb, rng):
    init_db.add_odst_models(mldb)
    init_db.add_cifar10(mldb)
    model = mldb.model(name='odst_resnet50_model')
    cifar10_test_small = mldb.dataset(name="cifar10-test").subsample(size=10, seed=923761329)
    eval = model.predict(dataset=cifar10_test_small, num_workers=0)
    acc = eval.accuracy
    assert acc >= 0.2
    
    model = mldb.model(name='odst_shakedroppyramid272_model')
    cifar10_test_small = mldb.dataset(name="cifar10-test").subsample(size=10, seed=923761329)
    eval = model.predict(dataset=cifar10_test_small, num_workers=0)
    acc = eval.accuracy
    assert acc >= 0.2
