"""Testing  fast neural learning rule functionality."""
import pytest
import sys

import numpy as np
import torch

from testbed import init_db
from testbed.learning_rule import LearningRule


def test_cpu():
    # silly synthetic data
    n = 100
    y = np.random.choice([0, 1], size=n)
    mu = np.random.randn(1, 32, 32, 3)
    mu = mu / np.linalg.norm(mu, axis=-1, keepdims=True)
    x = np.random.randn(n, 32, 32, 3) / np.sqrt(32 * 32 * 3)
    x += mu * (2 * y.reshape(-1, 1, 1, 1) - 1)
    
    rule = LearningRule(family='RandFeaturesCoatesNg_2to4')

    # test that network fits data
    model = rule.learn(examples=x, labels=y, device='cpu', verbose=True)
    eval = model.predict(examples=x, labels=y, device='cpu')
    print(f'Accuracy was {eval.accuracy}')
    assert eval.accuracy >= 0.9


@pytest.mark.gpu
def test_gpu(mldb):
    init_db.add_cifar10(mldb)

    rule = mldb.learning_rule(family='RandFeaturesCoatesNg_2to7')

    model = rule.learn(dataset='cifar10-train',
                       device='cuda:0',
                       verbose=True)
    eval = model.predict(dataset='cifar10-test')
    assert eval.accuracy >= 0.65


def test_score_subsampling(mldb, rng):
    n = 20
    x = rng.randn(n, 32, 32, 3)
    y = rng.choice([0, 1], size=n)

    dset = mldb.dataset(examples=x, labels=y, params={"classes_to_exclude": [0]})

    rule = mldb.learning_rule(family="RandFeaturesCoatesNg_2to1")

    model = rule.learn(dataset=dset, device="cpu", verbose=True)
    model_eval = model.predict(dataset=dset, device="cpu")
    assert np.allclose(model_eval.predictions, 1)
