"""Basic functionality tests for learning rules from SK Learn."""
import pytest
import sys

import numpy as np

from testbed.model_families.sklearn_models import get_model_families


@pytest.mark.parametrize(
    "family", list(get_model_families().keys())
)
def test_main(mldb, rng, family):
    """Ensure SKLearn models can initialize, memorize a small dataset, and store/load correctly."""
    examples = (256. * rng.rand(300, 32, 32, 3)).astype(np.uint8)
    labels = rng.randint(2, size=(300,))
    dataset = mldb.dataset(examples=examples, labels=labels)

    rule = mldb.learning_rule(family=family)
    model = rule.learn(dataset=dataset, name="model")
    evals1 = model.predict(dataset=dataset)
    evals1.store()

    rule_again = mldb.learning_rule(family=family)
    assert rule_again.uuid == rule.uuid

    model_again = mldb.model(name="model")
    evals2 = model_again.predict(dataset=dataset)

    assert evals1.accuracy >= 0.7
    assert np.array_equal(evals1.predictions, evals2.predictions)
