from tqdm import tqdm
import clip

import testbed
from testbed import init_db
from testbed.model_families.clip_zeroshot import ClipZSModel

# mldb = testbed.mldb.get_default_mldb()
mldb = testbed.mldb.get_local_mldb(testbed.mldb.CACHE_ROOT)
init_db.add_cifar10(mldb)
init_db.add_cifar10p1(mldb)
init_db.add_cifar10p2(mldb)


families = [fam for fam in mldb.model_registry.families.keys() if fam.startswith('clip_')]

def add_model(family):
    if family not in [x.name for x in mldb.get_models()]:
        model, preprocess = clip.load('ViT-B/32' if 'vit' in family else 'RN50')
        clip_model = ClipZSModel(model, preprocess)

        lr = mldb.learning_rule(family=family)
        lr.store()

        model_bytes = clip_model.serialize()
        mldb.import_model(learning_rule=lr,
                        model_bytes=model_bytes,
                        name=family)

        model = [x for x in mldb.get_models() if x.name == family]
        assert len(model) == 1
        model = model[0]
        model.store()

    model = mldb.model(name=family)
    eval_sets = set([x.test_set.name for x in model.get_evaluations()])
    for test_set in ['cifar10-test', 'cifar10.1-v6', 'cifar10.2-all', 'cinic10-imgnt-only-all', 'STL10', 'cifar10-STL10classes']:
        if test_set not in eval_sets:
            ev = model.predict(dataset=test_set, eval_batch_size=64)
            ev.store()
            print(f'accuracy {test_set}', ev.accuracy)


# for fam in families:
#     print(fam)
#     add_model(fam)
