from tqdm import tqdm
from testbed.mldb import get_default_mldb

mldb = get_default_mldb()
mldb.db.allow_hard_delete = True

def perm_delete_rels_blobs_and_obj(db_obj):
    for rel in mldb.db.get_relationship(first=db_obj):
        mldb.db.delete_relationship(rel, hide_only=False)
    for rel in mldb.db.get_relationship(second=db_obj):
        mldb.db.delete_relationship(rel, hide_only=False)
    for blob in mldb.db.get_blobs(db_obj, return_by='uuid'):
        mldb.db.delete_blob(blob, hide_only=False)
    mldb.db.delete(db_obj, hide_only=False)

def delete_model_and_all_associated(model):
    for checkpoint in model.checkpoints:
        for evaluation in checkpoint.as_model().get_evaluations():
            perm_delete_rels_blobs_and_obj(evaluation.db_obj)
        perm_delete_rels_blobs_and_obj(checkpoint.db_obj)
    for evaluation in model.get_evaluations():
        perm_delete_rels_blobs_and_obj(evaluation.db_obj)
    perm_delete_rels_blobs_and_obj(model.db_obj)


if __name__ == "__main__":
    noncompleted_models = [model for model in mldb.get_models(only_completed=False) if not model.train_completed]
    for model in tqdm(noncompleted_models):
        delete_model_and_all_associated(model)
