import pathlib
import shutil
import tempfile

import click
from sqlalchemy import select
import tqdm

import basicdb
from testbed import mldb


def chunks(lst, n):
    """Yield successive n-sized chunks from lst."""
    for i in range(0, len(lst), n):
        yield lst[i:i + n]


# A little hacky at the moment (should ideally be done at the basicdb level, not the mldb level), but works.
def export_inner(tmp_dir_name, output_name):
    mldb_postgres = mldb.get_default_mldb()
    mldb_sqlite = mldb.get_local_mldb(tmp_dir_name)
    with mldb_sqlite.db.db_adapter.engine.connect() as sqlite_conn:
        with mldb_postgres.db.db_adapter.engine.connect() as postgres_conn:
            for table in tqdm.tqdm(basicdb.sql_adapter.sqlalchemy_base.metadata.sorted_tables):
                data = [dict(row) for row in postgres_conn.execute(select(table.c))]
                for d_chunk in tqdm.tqdm(list(chunks(data, 100))):
                    sqlite_conn.execute(table.insert().values(d_chunk))
    shutil.move(pathlib.Path(tmp_dir_name) / 'db.sqlite', output_name)


@click.command()
@click.option('--output', type=str, default='postgres_export.sqlite', help='Output filename')
@click.option('--tmp_dir', type=str, default=None,
              help='Name of the temporary directory. Should not exist already. Uses Python\'s tempfile if not provided.')
def export(tmp_dir, output):
    if tmp_dir is None:
        with tempfile.TemporaryDirectory() as cur_tmp_dir_name:
            print(f'Created temporary directory {cur_tmp_dir_name} (will be removed at the end of the script)')
            export_inner(cur_tmp_dir_name, output)
    else:
        tmp_dir = pathlib.Path(tmp_dir)
        assert not tmp_dir.exists()
        tmp_dir.mkdir()
        print(f'Created temporary directory {tmp_dir} (will be removed at the end of the script)')
        export_inner(tmp_dir, output)
        shutil.rmtree(tmp_dir)


if __name__ == "__main__":
    export()