import json
import pathlib

res = {}

# manually select the wnids for CIFAR-10 classes with few ILSVRC-2012 wnids

# sorrel
res['horse'] = ['n02389026']

# airliner, wing, and military plane
res['airplane'] = ['n02690373', 'n04592741', 'n04552348']

# bullfrog, tailed frog, tree frog
res['frog'] = ['n01641577', 'n01644900', 'n01644373']

# ambulance, beach wagon, taxi, convertible, jeep, limo, race car, sports car
# NOT: minivan
res['automobile'] = ['n02701002', 'n02814533', 'n02930766', 'n03100240', 'n03594945', 'n03670208', 'n03777568', 'n04037443', 'n04285008']

# Egyptian cat, Persian cat, Siamese cat, tabby cat, tiger cat
res['cat'] = ['n02124075', 'n02123394', 'n02123597', 'n02123045', 'n02123159']

# fire truck, garbage truck, moving van, tow truck, trailer truck
# NOT included: ambulance, pick-up truck, police van, RV, minivan
res['truck'] = ['n03345487', 'n03417042', 'n03796401', 'n04461696', 'n04467665']

# gazelle, hartebeest, impala
res['deer'] = ['n02423022', 'n02422106', 'n02422699']

# aircraft carrier, container ship, fireboat, gondola, lifeboat,
# ocean liner, pirate ship, schooner, speedboat, trimaran
# yawl, canoe, catamaran
# NOT included: submarine, wreck, paddle wheel, amphibious vehicle
res['ship'] = ['n02687172', 'n03095699', 'n03344393', 'n03447447', 'n03662601',
               'n03673027', 'n03947888', 'n04147183', 'n04273569', 'n04483307',
               'n04612504', 'n02951358', 'n02981792']



# dog and bird have many wnids associated with them, so we get them out of the class hierarchy
# we used for the human accuracy project

def get_node_by_name(name, root):
    if root['name'] == name:
        return root
    if root['children'] is not None:
        for child in root['children'].values():
            cur_res = get_node_by_name(name, child)
            if cur_res is not None:
                return cur_res
    return None


def get_leaf_wnids(node):
    if node['children'] is None:
        return [node['wnid']]
    else:
        res = []
        for child in node['children'].values():
            res.extend(get_leaf_wnids(child))
        return res


with open(pathlib.Path(__file__).resolve().parent.parent / 'data' / 'imagenet_hierarchy.json', 'r') as f:
    data = json.load(f)

res['dog'] = list(set(get_leaf_wnids(get_node_by_name('dog', data['tree']))))
res['bird'] = list(set(get_leaf_wnids(get_node_by_name('bird', data['tree']))))


for k, v in res.items():
    print(f'Class {k}: {len(v)} wnids')


output_filename = pathlib.Path(__file__).resolve().parent.parent / 'data' / 'cifar10_ilsvrc2012_map.json'
with open(output_filename, 'w') as f:
    json.dump(res, f, sort_keys=True, indent=2)

print(f'\nWrote mapping to {output_filename}')