import os
import json

def create_mapping():
    synsets = sorted(os.listdir('anon'))
    with open('../data/cifar10_ilsvrc2012_map.json') as f:
        mapping = json.load(f)

    cifar10_idxs = []
    for cifar10_class in sorted(mapping.keys()):
        idxs = [synsets.index(x) for x in mapping[cifar10_class]]
        cifar10_idxs.append(idxs)
    
    print(cifar10_idxs)
    print(len(cifar10_idxs))
    print(mapping)

# create_mapping()


import torch.nn as nn
from tqdm import tqdm
from cnn_finetune import make_model

import testbed
from testbed import init_db
from testbed.model_families.pretrained_models import PretrainedModel, PretrainedModelFamilyRuleParams

# mldb = testbed.mldb.get_default_mldb()
mldb = testbed.mldb.get_local_mldb(testbed.mldb.CACHE_ROOT)
init_db.add_cifar10(mldb)
init_db.add_cifar10p1(mldb)
init_db.add_cifar10p2(mldb)


families = [fam for fam in mldb.model_registry.families.keys() if fam.startswith('pretrained_')]

def add_model(family):
    if family not in [x.name for x in mldb.get_models()]:
        net_name = family.replace('pretrained_', '')
        if 'vgg' in net_name or 'squeezenet' in net_name or 'alexnet' in net_name:
            model = make_model(net_name, num_classes=1000, pretrained=True, use_original_classifier=True, input_size=(224, 224))
        else:
            model = make_model(net_name, num_classes=1000, pretrained=True, use_original_classifier=True)
        info = model.original_model_info
        max_input_range = max(info.input_range)
        mean, std = [x / max_input_range for x in info.mean], [x / max_input_range for x in info.std]
        model = nn.Sequential(
            testbed.model_families.imagenet_models.NormalizationLayer(mean, std, info.input_space),
            model,
        )

        input_size = (max(info.input_size), max(info.input_size))
        hparams = PretrainedModelFamilyRuleParams(input_size = input_size)
        pretrained_model = PretrainedModel(model, hparams)

        lr = mldb.learning_rule(family=family)
        lr.store()

        model_bytes = pretrained_model.serialize()
        mldb.import_model(learning_rule=lr,
                        model_bytes=model_bytes,
                        name=family)

        model = [x for x in mldb.get_models() if x.name == family]
        assert len(model) == 1
        model = model[0]
        model.store()

    model = mldb.model(name=family)
    eval_sets = set([x.test_set.name for x in model.get_evaluations()])
    for test_set in ['cifar10-test', 'cifar10.1-v6', 'cifar10.2-all']:
        if test_set not in eval_sets:
            ev = model.predict(dataset=test_set, eval_batch_size=64)
            ev.store()
            print(f'accuracy {test_set}', ev.accuracy)


# for fam in tqdm(families):
#     print(fam)
#     add_model(fam)
